/*
 *  OpenGL 3.x Demonstration
 *  
 *  for the "Computergraphics 2" lab
 *
 *  Institute of Computer Graphics and Algorithms
 *  Vienna University of Technology
 *  Vienna, Austria
 *
 *  March 2010
 *
 *  ginquo
 */

// Note: If running OpenGL 2.1, you have to do the following to get it to run:
//       1. delete the two marked lines
//       2. rename 'in' to 'attribute'
//       3. rename 'out' to 'varying'

#version 130              // delete this line if using GLSL 1.2
precision mediump float;  // delete this line if using GLSL 1.2

// uniform shader-parameters
uniform vec3 light_position;
uniform vec4 light_color;
uniform vec4 ambient_color;
uniform vec4 specular_color;

uniform sampler2D shadow_map;

// the texture
uniform sampler2D colorTexture;

// fragment-shader input variables
//in vec4 frag_color;
in vec3 world_normal;
in vec4 world_position;
in vec2 texture_coordinates;
in vec4 SM_tex_coord;

// fragment-shader output variable (-> stored in the frame-buffer, i.e. "the pixel you see")
out vec4 fragColor; // ATI workaround


void main()
{

	float shininess = 1.0;

	// perform perspective division
	vec3 tex_coords = SM_tex_coord.xyz/SM_tex_coord.w;
	// read depth value from shadow map
	float depth = texture(shadow_map, tex_coords.xy).r;
	// perform depth comparison
	float inLight = (depth < tex_coords.z) ? 0.0 : 1.0;
	//inLight = 1.0;

	// renormalize and homogenize input variables
	vec3 normal = normalize(world_normal);
    vec3 position = world_position.xyz / world_position.w;
    
    // calculate the light-direction
    vec3 light_dir = normalize(light_position - position);

	// calculate lighting
  //  vec4 ambient = ambient_color * frag_color;
  //  vec4 diffuse = frag_color * light_color * max(0.0, dot(normal, light_dir));
  
	vec3 E = normalize(-position); // we are in Eye Coordinates, so EyePos is (0,0,0)  
    vec3 R = normalize(-reflect(light_dir,normal));  
    
    vec4 ambient = ambient_color;

    vec4 diffuse = light_color * max(0.0, dot(normal, light_dir));
    diffuse = clamp(diffuse, 0.0, 1.0);
    
    // calculate Specular Term:
    vec4 Ispec = specular_color * pow(max(dot(R,E),0.0),0.3*shininess);
     Ispec = clamp(Ispec, 0.0, 1.0); 

    
    fragColor = (ambient + diffuse * inLight) +  texture(colorTexture, texture_coordinates);
    
    /*
    float intensity = dot(light_dir, normal);
   
    
    vec4 color;
		
		//if (intensity > 0.95)
			//color = vec4(0.8,0.8,0.8,0.0);
		//else if (intensity > 0.5)
			//color = vec4(0.5,0.5,0.5,0.0);
		//else if (intensity > 0.25)
			//color = vec4(0.3,0.3,0.3,0.0);
		//else
			//color = vec4(0.1,0.1,0.1,0.0);
	
	color = vec4(intensity, intensity, intensity, 0.0);
    
    // write color to output
   fragColor = texture(colorTexture, texture_coordinates)+color/6;
   */
   
  //fragColor = color;
    //gl_FragColor = color;
    
    //gl_FragColor = vec4(0.6);
    
    // ATI workaround
    //fragColor = texture(colorTexture, texture_coordinates)+color/3;
    //gl_FragColor = texture2D( colorTexture, gl_TexCoord[0].st);
    
    // new calcs
    //vec4 texel = texture2D(colorTexture, texture_coordinates);
    //if (texel == vec4(0.0)) {
		//texel = vec4(0.4);
    //}
    //vec4 ambient = ambient_color * texel;
    //vec4 diffuse = texel * light_color * max(0.0, dot(normal, light_dir));
    
    // write color to output
    //gl_FragColor = ambient + diffuse;
    //gl_FragColor = vec4(texture_coordinates.s,texture_coordinates.t, 1.0, 1.0);
    //gl_FragColor = texture2D(colorTexture, vec2(0.0, 1.0))
	
}
